package controller

import (
	"github.com/gin-gonic/gin"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/device/service"
	"iot_manager_service/app/middleware"
	"iot_manager_service/util/common"
	"math"
	"net/http"
	"strconv"
)

// 一键报警终端表管理对象
var AlarmTerminal = new(alarmTerminalCtl)

type alarmTerminalCtl struct{}

func (c *alarmTerminalCtl) Detail(ctx *gin.Context) {
	id, e := strconv.Atoi(ctx.Query("id"))
	if e != nil {
		ctx.JSON(http.StatusOK, common.ParamsInvalidResponse(e.Error(), nil))
		return
	}

	device, err := service.AlarmTerminalService.Get(id)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, common.SuccessResponse(common.Succeeded, device))
}

func (c *alarmTerminalCtl) List(ctx *gin.Context) {
	searchValue := ctx.Query("searchValue")
	current, _ := strconv.Atoi(ctx.Query("current"))
	size, _ := strconv.Atoi(ctx.Query("size"))
	if current == 0 {
		current = 1
	}
	if size <= 0 || size > 100 {
		size = 10
	}
	devices, err := service.AlarmTerminalService.List(searchValue, current, size)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	pages := math.Ceil(float64(len(devices)) / float64(size))
	rsp := model.RsqAlarmTerminalList{
		Current: current,
		Size:    size,
		Total:   len(devices),
		Pages:   int(pages),
		Records: devices,
	}
	ctx.JSON(http.StatusOK, common.SuccessResponse(common.Succeeded, rsp))
}

func (c *alarmTerminalCtl) CreateOrUpdate(ctx *gin.Context) {
	value, _ := ctx.Get(middleware.Authorization)
	claims := value.(*middleware.Claims)

	var req dao.AlarmTerminal
	if err := ctx.ShouldBindJSON(&req); err != nil {
		ctx.JSON(http.StatusOK, common.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	err := service.AlarmTerminalService.CreateOrUpdate(claims.UserId, claims.TenantId, req)
	ctx.JSON(http.StatusOK, err)
}

func (c *alarmTerminalCtl) Remove(ctx *gin.Context) {
	value, _ := ctx.Get(middleware.Authorization)
	claims := value.(*middleware.Claims)

	var req *model.ReqAlarmTerminalRemove
	if err := ctx.ShouldBindJSON(&req); err != nil {
		ctx.JSON(http.StatusOK, common.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	err := service.AlarmTerminalService.Remove(claims.UserId, claims.TenantId, req.IDs)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, common.SuccessResponse(common.Succeeded, nil))
}

func (c *alarmTerminalCtl) ImportExcel(ctx *gin.Context) {
}

func (c *alarmTerminalCtl) ExportExcel(ctx *gin.Context) {
}

func (c *alarmTerminalCtl) ExportTemplate(ctx *gin.Context) {
}
