package dao

import (
	"gorm.io/gorm"
	"iot_manager_service/util/common"
	"time"
)

//InfoBoard信息基本屏
type InfoBoard struct {
	ID               int         `gorm:"primary_key" json:"id"`                            //编号
	InfoName         string      `gorm:"type:varchar(60)" json:"infoName"`                 //名称
	Sn               string      `gorm:"type:varchar(60)" json:"sn"`                       //设备序列号
	LampPoleId       int         `gorm:"type:int" json:"LampPoleId"`                       //所属灯杆id
	LampPoleName     string      `gorm:"type:varchar(64)" json:"lampPoleName"`             //灯杆名称
	LampPoleSn       string      `gorm:"type:varchar(64)" json:"lampPoleSn"`               //灯杆编码
	LampPoleLocation string      `gorm:"type:varchar(255)" json:"lampPoleLocation"`        //灯杆安装位置
	PoleLng          float64     `gorm:"type:double(17, 14) " json:"poleLng"`              //经度
	PoleLat          float64     `gorm:"type:double(17, 14)  " json:"poleLat"`             //纬度
	GatewayId        int         `gorm:"type:int" json:"gatewayId"`                        //所属网关id
	GroupId          int         `gorm:"type:int" json:"groupId"`                          //所属灯杆分组
	Resolution       int         `gorm:"type:int" json:"resolution"`                       //分辨率
	BrandId          int         `gorm:"type:int" json:"brandId"`                          //设备名
	ModelId          int         `gorm:"type:int" json:"modelId"`                          //设备型号
	InfoSize         int         `gorm:"type:int" json:"infoSize"`                         //信息屏尺寸
	RatedPower       float32     `gorm:"type:float(8, 2); default 0.00" json:"ratedPower"` //额定功率(LED灯)
	IPAddress        string      `gorm:"type:varchar(64)" json:"ipAddress"`                //IP地址
	BootStartTime    string      `gorm:"type:varchar(64)" json:"bootStartTime"`            //开机开始时间
	BootEndTime      string      `gorm:"type:varchar(64)" json:"bootEndTime"`              //开机结束时间
	NeverCloseDown   int         `gorm:"type:int" json:"neverCloseDown"`                   //永不关机：1-选择，2-取消
	InstallTime      common.Time `gorm:"type:date" json:"installTime"`                     //安装时间
	TenantId         int         `gorm:"type:int" json:"tenantId"`                         //租户ID
	CreateTime       time.Time   `gorm:"type:datetime" json:"createTime"`                  //新增时间
	CreateUser       int64       `gorm:"type:bigint" json:"createUser"`                    //新增记录操作用户ID
	UpdateTime       time.Time   `gorm:"type:datetime" json:"updateTime"`                  //修改时间
	UpdateUser       int64       `gorm:"type:bigint" json:"updateUser"`                    //修改用户
	IsDeleted        int         `gorm:"type:int;default 0" json:"isDeleted"`              //是否删除 0=未删除,1=删除
	Tag              string      `gorm:"type:varchar(255)" json:"tag"`                     //标签，保留字段(逗号区分)
	ExteriorUid      int         `gorm:"type:bigint" json:"exteriorUid"`                   //外设ID
	IsEnable         int         `gorm:"type:int; default 2 " json:"isEnable"`             //启用禁用：1启用，2禁用
}

func (InfoBoard) TableName() string {
	return "device_info_board"
}

func (c InfoBoard) Delete() error {
	return Db.Debug().Model(&c).Where("id = ?", c.ID).Updates(map[string]interface{}{"update_time": c.UpdateTime,
		"update_user": c.UpdateUser, "is_deleted": c.IsDeleted}).Error
}

func (c InfoBoard) IsExistedBySN() bool {
	var count int64
	_ = Db.Debug().Model(&c).Where("  sn = ? and is_deleted = ?",
		c.Sn, c.IsDeleted).Count(&count).Error
	return count > 0
}

func (c InfoBoard) IsExistedByNameAndCode() bool {
	var devices []InfoBoard
	err := Db.Debug().Model(&c).Where(" sn = ? and is_deleted = ?",
		c.Sn, c.IsDeleted).Find(&devices).Error
	//如果查询不到，返回相应的错误
	if gorm.ErrRecordNotFound == err {
		return false
	}
	for _, d := range devices {
		if d.ID != c.ID {
			return true
		}
	}
	return false
}

func (c *InfoBoard) Create() error {
	return Db.Debug().Model(&c).Save(&c).Error
}

func (c *InfoBoard) Update() error {
	return Db.Debug().Model(&c).Where(" id = ? ", c.ID).Updates(&c).Error
}

func (c *InfoBoard) GetDevice() error {
	err := Db.Debug().Model(&c).Where(" id = ? ", c.ID).Scan(&c).Error
	return err
}

func (c InfoBoard) GetDevices(offset, limit int) ([]InfoBoard, error) {
	var devices []InfoBoard
	err := Db.Debug().Model(&c).Where(" info_name like ? ", "%"+c.InfoName+"%").Offset(offset).Limit(limit).Find(&devices).Error
	return devices, err
}

func (c InfoBoard) GetAllDevices() ([]*InfoBoard, error) {
	var devices []*InfoBoard
	err := Db.Debug().Model(&c).Where(" tenant_id = ? and is_deleted = ? ", c.TenantId, c.IsDeleted).Scan(&devices).Error
	return devices, err
}

func (c InfoBoard) GetDevicesByGateway() []InfoBoard {
	var devices []InfoBoard
	Db.Debug().Model(&c).Where(" gateway_id = ? and is_deleted = 0",
		c.GatewayId).Find(&devices)
	return devices
}

func (c InfoBoard) GetDevicesByLampPole() []InfoBoard {
	var devices []InfoBoard
	Db.Debug().Model(&c).Where("lamp_pole_id = ? and is_deleted = 0",
		c.LampPoleId).Find(&devices)
	return devices
}

func (c InfoBoard) GetDevicesByResolution() []InfoBoard {
	var devices []InfoBoard
	Db.Debug().Model(&c).Where("resolution = ? and tenant_id = ? and is_deleted = 0",
		c.Resolution, c.TenantId).Find(&devices)
	return devices
}
