package service

import (
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util/cache"
	"iot_manager_service/util/common"
	"iot_manager_service/util/logger"
	"time"
)

// 中间件管理服务
var BridgeSensorService = new(bridgeSensorService)

type bridgeSensorService struct{}

func (s *bridgeSensorService) Get(id int) (*dao.BridgeSensor, *common.Errors) {
	// 创建查询实例
	device := &dao.BridgeSensor{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	return device, nil
}

func (s *bridgeSensorService) CreateOrUpdate(userId int64, tenantId int, req dao.BridgeSensor) *common.Errors {
	// 创建查询实例
	device := req
	device.TenantID = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			logger.Logger.Errorf("Create GetDeviceID err \n")
			return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			logger.Logger.Errorf("Create err = %s \n", err.Error())
			return common.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, common.OperationCreate, common.ModuleTypeDevice,
			common.DeviceTypeBridgeSensor, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
		return common.SuccessResponse(common.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		logger.Logger.Errorf("Update err = %s \n", err.Error())
		return common.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, common.OperationUpdate, common.ModuleTypeDevice,
		common.DeviceTypeBridgeSensor, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
	return common.SuccessResponse(common.Succeeded, nil)
}

func (s *bridgeSensorService) List(searchValue string, current, size int) ([]dao.BridgeSensor, *common.Errors) {
	device := dao.BridgeSensor{}
	if searchValue != "" {
		device.SN = searchValue
		device.Name = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *bridgeSensorService) Remove(userId int64, tenantId int, id int) *common.Errors {
	// 创建查询实例
	device := &dao.BridgeSensor{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return common.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, common.OperationRemove, common.ModuleTypeDevice,
		common.DeviceTypeBridgeSensor, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
	return nil
}

func (s *bridgeSensorService) GetList(tenantId int) ([]*dao.BridgeSensor, *common.Errors) {
	var devices []*dao.BridgeSensor
	err := cache.Redis.Get(getTransformerListRedisKey(tenantId)).Scan(&devices)
	if err == nil {
		return devices, nil
	}

	device := &dao.BridgeSensor{
		TenantID:  tenantId,
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	_ = cache.Redis.Set(getTransformerListRedisKey(tenantId), devices, 0).Err()

	return devices, nil
}

func (s *bridgeSensorService) Enable(id, status int) *common.Errors {
	// 创建查询实例
	device := &dao.BridgeSensor{
		ID:       id,
		IsEnable: status,
	}

	err := device.UpdateEnable()
	if err != nil {
		return common.FailResponse(err.Error(), nil)
	}
	return nil
}
