package service

import (
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util/common"
	"iot_manager_service/util/logger"
	"time"
)

var BridgeService = new(bridgeService)

type bridgeService struct{}

func (s *bridgeService) Get(id int) (*dao.Bridge, *common.Errors) {
	// 创建查询实例
	device := &dao.Bridge{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	return device, nil
}

func (s *bridgeService) CreateOrUpdate(userId int64, tenantId int, req dao.Bridge) *common.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			logger.Logger.Errorf("Create GetDeviceID err \n")
			return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			logger.Logger.Errorf("Create err = %s \n", err.Error())
			return common.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, common.OperationCreate, common.ModuleTypeDevice,
			common.DeviceTypeBridge, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
		return common.SuccessResponse(common.Succeeded, nil)
	}

	if device.IsExistedByNameAndCode() {
		return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		logger.Logger.Errorf("Update err = %s \n", err.Error())
		return common.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, common.OperationUpdate, common.ModuleTypeDevice,
		common.DeviceTypeBridge, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
	return common.SuccessResponse(common.Succeeded, nil)
}

func (s *bridgeService) List(searchValue string, current, size int) ([]dao.Bridge, *common.Errors) {
	device := dao.Bridge{}
	if searchValue != "" {
		device.SN = searchValue
		device.Name = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *bridgeService) Remove(userId int64, tenantId int, id int) *common.Errors {
	// 创建查询实例
	device := &dao.Bridge{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return common.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, common.OperationRemove, common.ModuleTypeDevice,
		common.DeviceTypeBridge, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
	return nil
}

func (s *bridgeService) GetList(tenantId int) ([]*dao.Bridge, *common.Errors) {
	// todo use redis cache
	device := &dao.Bridge{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}

	return devices, nil
}
