package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util/cache"
	"iot_manager_service/util/common"
	"iot_manager_service/util/logger"
	"time"
)

var GarbageService = new(garbageService)

type garbageService struct{}

func (s *garbageService) Get(id int) (*dao.Garbage, *common.Errors) {
	// 创建查询实例
	device := &dao.Garbage{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *garbageService) CreateOrUpdate(userId int64, tenantId int, req dao.Garbage) *common.Errors {
	// 创建查询实例
	device := req
	device.UpdateUser = userId
	device.TenantId = tenantId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			logger.Logger.Errorf("Create GetDeviceID err \n")
			return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			logger.Logger.Errorf("Create err = %s \n", err.Error())
			return common.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, common.OperationCreate, common.ModuleTypeDevice,
			common.DeviceTypeGarbage, common.GetDeviceObject(device.ID, device.DeviceName), common.OperationSuccess)
		return common.SuccessResponse(common.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		logger.Logger.Errorf("Update err = %s \n", err.Error())
		return common.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, common.OperationUpdate, common.ModuleTypeDevice,
		common.DeviceTypeGarbage, common.GetDeviceObject(device.ID, device.DeviceName), common.OperationSuccess)
	return common.SuccessResponse(common.Succeeded, nil)
}

func (s *garbageService) List(searchValue string, current, size int) ([]dao.Garbage, *common.Errors) {
	device := dao.Garbage{}
	if searchValue != "" {
		device.DeviceSN = searchValue
		device.DeviceName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *garbageService) Remove(userId int64, tenantId int, id int) *common.Errors {
	// 创建查询实例
	device := &dao.Garbage{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return common.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, common.OperationRemove, common.ModuleTypeDevice,
		common.DeviceTypeGarbage, common.GetDeviceObject(device.ID, device.DeviceName), common.OperationSuccess)
	return nil
}

func (s *garbageService) GetList(tenantId int) ([]*dao.Garbage, *common.Errors) {
	var devices []*dao.Garbage
	err := cache.Redis.Get(getGarbageListRedisKey(tenantId)).Scan(&devices)
	if err == nil {
		return devices, nil
	}
	device := &dao.Garbage{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	_ = cache.Redis.Set(getAlarmListRedisKey(tenantId), devices, 0).Err()
	return devices, nil
}

func getGarbageListRedisKey(tenantId int) string {
	return fmt.Sprintf(model.GarbageList, tenantId)
}

func (s *garbageService) GetAll() []*dao.Garbage {
	device := &dao.Garbage{}
	devices := device.GetAll()
	return devices
}
