package service

import (
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util/cache"
	"iot_manager_service/util/common"
	"iot_manager_service/util/logger"
	"time"
)

var InfoBoardService = new(infoBoardService)

type infoBoardService struct{}

func (s *infoBoardService) Get(id int) (*model.InfoBoardDetail, *common.Errors) {
	// 创建查询实例
	device := &dao.InfoBoard{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	endTime, state := cache.GetDeviceState(device.Sn)
	return &model.InfoBoardDetail{InfoBoard: *device,
		RunState:     state,
		NetworkState: state,
		EndLineTime:  endTime,
	}, nil
}

func (s *infoBoardService) CreateOrUpdate(userId int64, tenantId int, req dao.InfoBoard) *common.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			logger.Logger.Errorf("Create GetDeviceID err \n")
			return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			logger.Logger.Errorf("Create err = %s \n", err.Error())
			return common.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, common.OperationCreate, common.ModuleTypeDevice,
			common.DeviceTypeInfoBoard, common.GetDeviceObject(device.ID, device.InfoName), common.OperationSuccess)
		return common.SuccessResponse(common.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		logger.Logger.Errorf("Update err = %s \n", err.Error())
		return common.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, common.OperationUpdate, common.ModuleTypeDevice,
		common.DeviceTypeInfoBoard, common.GetDeviceObject(device.ID, device.InfoName), common.OperationSuccess)
	return common.SuccessResponse(common.Succeeded, nil)
}

func (s *infoBoardService) List(searchValue string, current, size int) ([]model.InfoBoardDetail, *common.Errors) {
	device := dao.InfoBoard{}
	if searchValue != "" {
		device.Sn = searchValue
		device.InfoName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	var details []model.InfoBoardDetail
	for _, d := range devices {
		endTime, state := cache.GetDeviceState(d.Sn)
		details = append(details, model.InfoBoardDetail{
			InfoBoard:    d,
			RunState:     state,
			NetworkState: state,
			EndLineTime:  endTime,
		})
	}
	return details, nil
}

func (s *infoBoardService) Remove(userId int64, tenantId int, id int) *common.Errors {
	// 创建查询实例
	device := &dao.InfoBoard{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return common.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, common.OperationRemove, common.ModuleTypeDevice,
		common.DeviceTypeInfoBoard, common.GetDeviceObject(device.ID, device.InfoName), common.OperationSuccess)
	return nil
}

func (s *infoBoardService) GetByGateway(gatewayId int) []dao.InfoBoard {
	device := &dao.InfoBoard{
		GatewayId: gatewayId,
	}
	return device.GetDevicesByGateway()
}

func (s *infoBoardService) GetByLampPole(lampPoleId int) []dao.InfoBoard {
	device := &dao.InfoBoard{
		LampPoleId: lampPoleId,
	}
	return device.GetDevicesByLampPole()
}

func (s *infoBoardService) GetByResolution(tenantId, resolution int) []dao.InfoBoard {
	device := &dao.InfoBoard{
		Resolution: resolution,
		TenantId:   tenantId,
	}
	return device.GetDevicesByResolution()
}
